extern crate clap;
extern crate rouille;

use std::{
    any::Any,
    fs,
    path::Path,
    sync::{Arc, Mutex},
};

use anyhow::{anyhow, Result};
use clap::{Arg, ArgAction, Command};
use rouille::Response;

fn get_config(path: &Path) -> Vec<String> {
    let c = fs::read_to_string(path)
        .expect("Config file does not exist")
        .split("\n")
        .into_iter()
        .map(|d| d.trim())
        .filter(|d| d.len() > 0)
        .map(|d| d.to_string())
        .collect::<Vec<String>>();

    if c.len() == 0 {
        panic!("Config seams to be empty")
    }

    return c;
}

fn switch_to_config(base_path: &String, p: String) -> Result<()> {
    let res = std::process::Command::new("ln")
        .arg("-sf")
        .arg(base_path.clone().to_owned() + "/" + &p + ".conf")
        .arg(base_path.clone().to_owned() + "/monitors.conf")
        .spawn();

    if res.is_err() {
        println!("Well somwthing went very wrong");
        return Err(anyhow!("Something went very wrong"));
    }

    let res2 = std::process::Command::new("hyprctl").arg("reload").spawn();

    if res2.is_err() {
        println!("Was able to change the link but could not reload hyprland");
        return Err(anyhow!("Something went very wrong"));
    }

    return Ok(());
}

fn main() {
    println!("Hello, world!");

    let matches = Command::new("Monitor Controller")
        .about("Monitor Contorller server/cli")
        .arg(
            Arg::new("daemon")
                .short('d')
                .help("Runs the app in daemon mode!")
                .action(ArgAction::SetTrue),
        )
        .arg(
            Arg::new("config")
                .short('c')
                .long("config")
                .help("Gives the path for the config location of the list of configs"),
        )
        .arg(
            Arg::new("base_path")
                .short('p')
                .help("Gives the base path for the configs"),
        )
        .arg(
            Arg::new("change_on_boot")
                .short('b')
                .help("Change to the first option when the daemon starts")
                .action(ArgAction::SetTrue),
        )
        .arg(Arg::new("switch_to").short('s').action(ArgAction::Set))
        .get_matches();

    let daemon_mode: bool = matches
        .get_one::<bool>("daemon")
        .unwrap()
        .clone()
        .to_owned();

    if daemon_mode {
        println!("Paying attention on 11111");

        let base_path = matches.get_one::<String>("base_path");
        if base_path.is_none() {
            println!("Base path is required when running daemon mode");
            return;
        }

        let base_path = base_path.unwrap().clone().to_owned();

        let config_path = matches.get_one::<String>("config");

        if config_path.is_none() {
            println!("Config path is required when running daemon mode");
            return;
        }

        let config_path = Path::new(&config_path.unwrap())
            .canonicalize()
            .ok()
            .unwrap();

        let config = Arc::new(Mutex::new(get_config(&config_path)));

        let change_on_boot: bool = matches
            .get_one::<bool>("daemon")
            .unwrap()
            .clone()
            .to_owned();

        if change_on_boot {
            let c = config.clone();
            let c = c.lock().ok().unwrap();

            let u = c[0].clone().to_owned();

            std::process::Command::new("ln")
                .arg("-sf")
                .arg(base_path.clone().to_owned() + "/" + &u.to_string() + ".conf")
                .arg(base_path.clone().to_owned() + "/monitors.conf")
                .spawn()
                .expect("Failed to change monitor on boot");
        }

        let config_ptr: Arc<Mutex<usize>> = Arc::new(Mutex::new(0));

        rouille::start_server("0.0.0.0:11111", move |request| {
            if request.url().eq("/reload") {
                println!("Reloading config!");

                let c = config.clone();
                let mut c = c.lock().ok().unwrap();
                *c = get_config(&config_path);

                return Response::text("reloaded!");
            } else if request.url().eq("/next_config") {
                println!("Switiching to the next config");
                let c_ptr = config_ptr.clone();
                let mut c_ptr = c_ptr.lock().ok().unwrap();

                let c = config.clone();
                let c = c.lock().ok().unwrap();

                let ptr = (*c_ptr + 1) % c.len();

                if switch_to_config(&base_path, c[ptr].clone()).is_err() {
                    return Response::text("Something went very wrong!").with_status_code(500);
                }

                *c_ptr = ptr;
                return Response::text("Change it!");
            }

            if !request.url().starts_with("/switch_to/") {
                return Response::text("Not reallllly sure what you are trying to do :3");
            }

            let u = request.url().replace("/switch_to/", "");

            let c = config.clone();
            let c = c.lock().ok().unwrap();

            let found_it = c.iter().any(|d| d.eq(&u));

            if !found_it {
                println!("Could not find the requested url");
                return Response::text("Could not find the requested config!")
                    .with_status_code(404);
            }

            if switch_to_config(&base_path, u.to_string()).is_err() {
                return Response::text("Something went very wrong!").with_status_code(500);
            }

            let c_ptr = config_ptr.clone();
            let mut c_ptr = c_ptr.lock().ok().unwrap();

            *c_ptr = c.iter().position(|d| d.eq(&u)).unwrap();

            return Response::text("Chaged it");
        })
    } else {
        let possible_switch: Option<&String> = matches.get_one("switch_to");

        if possible_switch.is_none() {
            println!("You have to provide an configuration to switch to!");
            return;
        }

        let possible_switch = possible_switch.unwrap();

        println!("Trying to switch: {}", possible_switch);

        let resp = reqwest::blocking::get(format!(
            "http://localhost:11111/switch_to/{}",
            possible_switch
        ));

        if resp.is_err() {
            println!("Failed to change the configuration");
            return;
        }

        let resp = resp.ok().unwrap().text().ok().unwrap();

        println!("{}", resp);
    }
}
